package Algorithm::SetSimilarity;
use 5.008005;
use strict;
use warnings;

our $VERSION = "0.0.0_03";

1;
__END__

=encoding utf-8

=head1 NAME

Algorithm::SetSimilarity - It's new $module

=head1 SYNOPSIS

    use Algorithm::SetSimilarity;

=head1 DESCRIPTION

Algorithm::SetSimilarity is ...

=head1 LICENSE

Copyright (C) Toshinori Sato (@overlast).

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

Toshinori Sato (@overlast) E<lt>overlasting@gmail.comE<gt>

=cut
